/*************************************************************************
Autor:   Martin Wende <Digifreak@gmx.de>   http://fritzler-avr.de
Software: WinAVR 01.10.2010
Hardware: ATmega 
License:  GNU General Public License 
          
LICENSE:
    Copyright (C) 2011 Martin Wende

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/


/*
Hier befinden sich alle Prozeduren fr den Umgang mit dem GMS Modul.

Ausgabe der Antworten: (Standard)
1. Echo des Befehls 
2. Antwort auf den Befehl
3. OK oder ERROR

Ausgabe der Antworten: (auf die sich diese Bibliothek bezieht)
1. Antwort auf den Befehl
2. OK oder ERROR

Daher muss zuerst das Echo Abgeschalten werden, wird alles in init_GSM erledigt.
*/

#include <avr/pgmspace.h>
#include <util/delay.h>
#include <string.h>

#include <gsm_strings.c>
#include <GSM.h>
#include <uart.h>

#define wait 10

//Statusvariablen
uint8_t ring = 0;
uint8_t no_carrier = 0;
uint8_t in_sms = 0;
char sms_index [3] = "\0";

void delay(void) {
	//_delay_ms(1);
}


//*****************************UART String Geraffel**************************************

//Prozedur zum einbinden anderer UART Librarys
uint16_t ser_in(uint8_t ziel){
	return uart_getc();
}

/** \brief pollt den UART Eingang wenn die GSM Lib auf eine Antwort wartet

	Die Baudrate muss >= 9600Baud sein <br>
	Ein Zeichen ist geliefert sofern vorhanden, ist keins im Empfangspuffer wird 10000us lang abgefragt.
*/
uint8_t uart_pollen (uint8_t ziel) {//fr minimal 9600baud!
	uint16_t empfangen = 0;
	uint16_t waited = 0;
	uint8_t data = 0;
	uint8_t status = 1;
	while (status != 0) {
		empfangen = ser_in(ziel);
		status = (empfangen>>8);
		_delay_us(10);
		if (waited > 1000) {
			break;
		}
		waited++;
	}
	data = empfangen;
	return data;
}

//Der max. 3 stellige Index der grade eingegangenen SMS ist geliefert.
uint8_t parse_sms_index(const char *String) {
	uint8_t i = 0;
	uint8_t chara = 0;
	while ((String[i] != ',') && (i < 15)) { //String durchlaufen bis , erreicht
		i++;
	}
	i++;//eine Stelle nach , anfangen
	chara = String[i];//SMS Index einfangen <- noch umbauen auf ein 10er array
	return chara;
}	

uint8_t get_GSM(char *String, uint8_t maxchar) {
	uint16_t empfangen = 0;
	uint8_t chara = 0;
	uint8_t status = 0;
	uint8_t count = 0;
	empfangen = ser_in(GSM);
	status = (empfangen>>8);
	chara = empfangen;
	if ((status != 1) && (chara != 13)){//nachsehen ob Zeichen vorhanden und wenn ja ob \r
		return 0;
	}
	chara = uart_pollen(GSM);
	if (chara != 10) {//nachsehen ob \n zeichen
		return 0;
	}//hier angekommen kann der eigentliche String eingelesen werden, bis \r
	chara = uart_pollen(GSM);
	
	//abfangen der zwischenmeldungen error noch einbauen
	if (chara == 'R') { //RING empfangen
		char tmp3[5] = "\0";
		uint8_t i = 0;
		while (chara != 13) {
			tmp3[i] = chara;
			i++;
			chara = uart_pollen(GSM);
		}
		if (strcmp_P(tmp3,RING) == 0) {
			ring = 1;
			chara = uart_pollen(GSM);//noch \n Empfangen und verwerfen
			return 0;
		}
	}
	if (chara == 'N') { //NO_CARRIER empfangen
		char tmp3[11] = "\0";
		uint8_t i = 0;
		while (chara != 13) {
			tmp3[i] = chara;
			i++;
			chara = uart_pollen(GSM);
		}
		if (strcmp_P(tmp3,NO_CARRIER) == 0) {
			no_carrier = 1;
			chara = uart_pollen(GSM);//noch \n Empfangen und verwerfen
			return 0;
		}
	}
	if (chara == '+') { //befehl parsen
		char tmp3 [20] = "\0";
		while (chara != ':') {//Bis Befehlsende parsen
			tmp3[count] = chara;
			count++;
			chara = uart_pollen(GSM);
		}

		if (strcmp_P(tmp3,CMTI) == 0) {//wenn sms empfangen hier aufhren
			in_sms = 1;
			while (chara != 13) {//restlichen string abrufen fr sms index
				tmp3[count] = chara;
				count++;
				chara = uart_pollen(GSM);
			}
			chara = uart_pollen(GSM);//noch \n Empfangen und verwerfen
			sms_index[0] = parse_sms_index((char*)&tmp3);
			return 0;
		}
	}
	
	while ((count < (maxchar-1)) && (chara != 13)) {
		String[count] = chara;
		count++;
		chara = uart_pollen(GSM);
	}
	String[count] = '\0'; //String terminieren
	//falls String zu lang, noch Rest empfangen und verwerfen
	if (count >= (maxchar-1)) {
		while (chara != 13) {
			chara = uart_pollen(GSM);
		}
	}
	chara = uart_pollen(GSM);//noch \n Empfangen und verwerfen
	return 1;
}

//Der String aus dem ROM ist inkl. Terminierung zum GSM Modul gesendet.
void put_GSM_P(const char *String) {
	while (pgm_read_byte(String) != 0) {
		ser_out(pgm_read_byte(String++), GSM);
	}
	ser_out(13, GSM);
	ser_out(10, GSM);
}

//Der String ist inkl. Terminierung zum GSM Modul gesendet.
void put_GSM(const char *String) {
	ser_out_str(String, GSM);
	ser_out(13, GSM);
	ser_out(10, GSM);
}

//Der Uart Empangspuffer fr GSM ist geleert.
//Dies ist ntig, weil das Modul beim starten undefinierte Zeichen ausgibt.
void empty_ser_in(void) { 
	uint16_t empfangen = 0;
	uint8_t status = 0;
	while (status == 0) {
		empfangen = ser_in(GSM);
		status = (empfangen>>8);
	}
}

void off_GSM(void) {
	//GSM Modul abschalten
	if ((PIN(STATUS_PORT) & (1<<STATUS_PIN))>>STATUS_PIN) {//berspringen falls bereits abgeschalten
		PWRKEY_PORT |= (1<<PWRKEY_PIN); //auf low ziehen
		_delay_ms(500);
		_delay_ms(500);
		_delay_ms(500);
		_delay_ms(500);
		PWRKEY_PORT &= ~(1<<PWRKEY_PIN); //loslassen
		while ((PIN(STATUS_PORT) & (1<<STATUS_PIN))>>STATUS_PIN) { //warten bis gestartet
			_delay_us(20);
		}
	}
}	

void init_GSM(void) {
	char tmp[3] = "\0";
	uint8_t stat = 0;
	uint8_t i = 0;
	
	DDR(PWRKEY_PORT) |= (1<<PWRKEY_PIN);
	DDR(STATUS_PORT) &= ~(1<<STATUS_PIN);
	
	uart_init(UBRR_REG);

	//GSM Modul einschalten
	if (((PIN(STATUS_PORT) & (1<<STATUS_PIN))>>STATUS_PIN) == 0) {//berspringen falls bereits eingeschalten
		PWRKEY_PORT |= (1<<PWRKEY_PIN); //auf low ziehen
		_delay_ms(500);
		_delay_ms(500);
		_delay_ms(500);
		_delay_ms(500);
		PWRKEY_PORT &= ~(1<<PWRKEY_PIN); //loslassen
		while ((PIN(STATUS_PORT) & (1<<STATUS_PIN))>>STATUS_PIN) {
			_delay_us(20);
		}
		stat = 1;
	}
	_delay_ms(100);
	empty_ser_in();
	//AT Interpreter pollen bis OK
	while (AT_active() != 1) {
		;
	}
	_delay_ms(100);
	put_GSM_P(ATE0);//Echo abschalten
	_delay_ms(100);
	put_GSM_P(AT_IPR_38400);//baudrate einstellen
	_delay_ms(100);
	put_GSM_P(AT_CMGF_1); //SMS im textmode
	_delay_ms(100);
	empty_ser_in();
	if (stat == 1) { //nur aufrufen wenn gsm modul frisch eingeschaltet, sonst freeze
		while (get_GSM((char*)&tmp, 3) == 0) {//CFUN abfangen
			i++;
			if (i >= 250) {goto ex;}
		}	
		i = 0;
		while (get_GSM((char*)&tmp, 3) == 0) {//CPIN abfangen
			i++;
			if (i >= 250) {goto ex;}
		}
	}
	ex:;
	empty_ser_in();
}

//***************************Profile Commands******************************

//Vor.: UART init und GSM init.
//Eff.: Ist der AT Interpreter aktiv ist 1 geliefert sonst 0.
uint8_t AT_active(void) {
	char String[3] = "\0";
	put_GSM_P(AT);
	_delay_us(100);
	while (get_GSM((char*)&String, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(String , OK) == 0) {return 1;}
	else {return 0;}
}

//Vor.: UART init und GSM init.
//Eff.: Der Name des GSM Moduls ist geliefert.	
uint8_t GSM_Modul(char *String, uint8_t maxchar) {
	char tmp[3] = "\0";
	empty_ser_in();
	put_GSM_P(ATI);
	delay();
	while (get_GSM(String, maxchar) == 0) {//auf Antwort warten
		;
	}
	while (get_GSM((char*)&tmp, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp , OK) == 0) {return 1;}
	else {return 0;}
}

//**************************General Commands*******************************

//Vor.: UART init und GSM init.
//Eff.: Der Netzstatus ist geliefert.
//      0: fehlgeschlagen
//      1: eingebucht, Heimatnetz
//      2: nicht eingebucht, Netzsuche
//      3: nicht eingebucht, Einbuchung abgelehnt
//      4: Status unbekannt
//      5: eingebucht, Fremdnetz
uint8_t Netzstatus (void) {
	char tmp[20] = "\0";
	char tmp2[3] = "\0";
	uint8_t i = 0;
	uint8_t chara = 0;
	empty_ser_in();
	put_GSM_P(AT_CREG);
	delay();
	while (get_GSM((char*)&tmp, 20) == 0) {//auf Antwort warten
		;
	}	
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}	
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	while ((tmp[i] != ',') && (i < 21)) { //String durchlaufen bis , erreicht
		i++;
	}
	i++;//eine Stelle nach , anfangen
	chara = tmp[i];//Netzstatus einfangen
	chara = chara - 48; //in int umwandeln
	return chara;
}
	


//Vor.: UART init und GSM init. Das Modul muss eingeloggt sein!
//Eff.: Der Name des Providers ist geliefert.	
uint8_t Provider(char *String) {
	char tmp[41] = "\0";
	char tmp2[3] = "\0";
	uint8_t i = 0;
	uint8_t anf = 0;
	empty_ser_in();
	put_GSM_P(AT_COPS);
	delay();
	while (get_GSM((char*)&tmp, 41) == 0) {//auf Antwort warten
		;
	}	
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	while ((tmp[i] != 34) && (i < 42)) { //String durchlaufen bis " erreicht
		i++;
	}
	i++;//eine Stelle nach 1. " anfangen
	anf = i;
	while ((tmp[i] != 34) && (i < 42)) {//String kopieren bis zum nchsten "
		String[i-anf] = tmp[i];
		i++;
	}
	if ( i == 42) {
		String[0] = '\0';
		return 0;
	}
	String[i-anf] = '\0';
	return 1;
}

//Vor.: UART init und GSM init. 
//Eff.: Der Empfnagsindex ist geliefert.	
// 0 - weniger als -115dBm
// 1 - -111dBm
// 2...30 - -110...-54dBm
// 31 - mehr als -52dBm
// 99 - unbekannt
uint8_t Empfang(char *String) {
	char tmp[41] = "\0";
	char tmp2[3] = "\0";
	uint8_t i = 0;
	uint8_t anf = 0;
	empty_ser_in();
	put_GSM_P(AT_CSQ);
	delay();
	while (get_GSM((char*)&tmp, 41) == 0) {//auf Antwort warten
		;
	}	
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	while ((tmp[i] != ' ') && (i < 42)) { //String durchlaufen bis LEER erreicht
		i++;
	}
	i++;//eine Stelle nach 1. " anfangen
	anf = i;
	while ((tmp[i] != ',') && (i < 42)) {//String kopieren bis zum nchsten ,
		String[i-anf] = tmp[i];
		i++;
	}
	if ( i == 42) {
		String[0] = '\0';
		return 0;
	}
	String[i-anf] = '\0';
	return 1;
}

//****************************Telefoncommands***************

//Vor.: UART init und GSM init. Das Modul muss eingeloggt sein!
//Eff.: 1 ist geliefert bei Anruf, sonst 0
uint8_t get_ring(void){
	uint8_t out = ring;
	if (out){
		ring = 0;
		return out;
	}
	return out;
}

//Vor.: UART init und GSM init. Das Modul muss eingeloggt sein!
//Eff.: Die Telefonnumemr des anrufers ist geliefert. 
//      Der String ist leer	bei unterdrckter Nummer. Maximale lnge: 20 zeichen.
uint8_t get_ringingnumber(char *String) {
	char tmp[40] = "\0";
	char tmp2[3] = "\0";
	uint8_t i = 0;
	uint8_t anf = 0;
	empty_ser_in();
	put_GSM_P(AT_CLCC);
	delay();
	while (get_GSM((char*)&tmp, 40) == 0) {//auf Antwort warten
		;
	}
	if (strcmp_P(tmp , OK) != 0) {//abgefragt ohne anrufer
		String[0] = 0;
		return 0;
	}	
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	while ((tmp[i] != 34) && (i < 40)) { //String durchlaufen bis " erreicht
		i++;
	}
	i++;//eine Stelle nach 1. " anfangen
	anf = i;
	while ((tmp[i] != 34) && (i < 40)) {//String kopieren bis zum nchsten "
		String[i-anf] = tmp[i];
		i++;
	}
	if ( i == 21) {
		String[0] = '\0';
		return 0;
	}
	String[i-anf] = '\0';
	return 1;
}

//Vor.: UART init und GSM init. Das Modul muss eingeloggt sein!
//Eff.: Die Name des Anrufers ist geliefert, insofern er im Telefonbuch steht. 
//      Der String ist leer	bei keinem auffindbaren Eintrag. Maximale lnge: 16 zeichen.
uint8_t get_anrufer(char *String) {	
	char tmp[60] = "\0";
	char tmp2[3] = "\0";
	uint8_t i = 0;
	uint8_t anf = 0;
	empty_ser_in();
	put_GSM_P(AT_CLCC);
	delay();
	while (get_GSM((char*)&tmp, 60) == 0) {//auf Antwort warten
		;
	}
	if (strcmp_P(tmp , OK) != 0) {//abgefragt ohne anrufer
		String[0] = 0;
		return 0;
	}	
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 1. " erreicht
		i++;
	}
	i++;//eine Stelle nach 1. " weitermachen
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 2. " erreicht
		i++;
	}
	i++;//eine Stelle nach 2. " weitermachen
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 3. " erreicht
		i++;
	}
	i++;//eine Stelle nach 3. " anfangen
	anf = i;
	while ((tmp[i] != 34) && (i < 60)) {//String kopieren bis zum nchsten "
		String[i-anf] = tmp[i];
		i++;
	}
	if ( i == 61) {
		String[0] = '\0';
		return 0;
	}
	String[i-anf] = '\0';
	return 1;
}

//Vor.: UART init und GSM init. Das Modul muss eingeloggt sein und ein Anruf anliegen.
//Eff.: Der Anruf ist aufgelegt.
uint8_t auflegen(void) {
	char tmp[3] = "\0";
	empty_ser_in();
	put_GSM_P(ATH);
	delay();
		while (get_GSM((char*)&tmp, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp(tmp , OK) != 0) {return 0;}	
	return 1;
}

//Vor.: UART init und GSM init. Das Modul muss eingeloggt sein und ein Anruf anliegen.
//Eff.: Der Anruf ist angenommen.
uint8_t annehmen(void) {
	char tmp[3] = "\0";
	empty_ser_in();
	put_GSM_P(ATA);
	delay();
		while (get_GSM((char*)&tmp, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp , OK) != 0) {return 0;}	
	return 1;
}

//Vor.: UART init und GSM init. Das Modul muss eingeloggt sein.
//Vor.: Die Nummer muss als String im Format +49xxxx vorliegen.
//Eff.: Der Anruf ist angenommen.
uint8_t anrufen(char* Nummer) {
	char tmp[3] = "\0";
	empty_ser_in();
	ser_out_str_P(ATD, GSM);
	ser_out_str(Nummer, GSM);
	put_GSM(";");
	delay();
		while (get_GSM((char*)&tmp, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp , OK) != 0) {return 0;}	
	return 1;
}


//*************************Telefonbuch*******************

//Vor.: UART init und GSM init.
//      Ein Index (1 - 230), eine Telefonnummer nach 145 (+49xxx) (max. 20 Zeichen), ein Titel (max. 16 Zeichen).
//Eff.: Der Eintrag ist ins Telefonbuch auf der SIMKarte geschrieben.
uint8_t write_entry(const char* Index, const char* Nummer, const char* Titel) {
	char tmp[3] = "\0";
	empty_ser_in();
	ser_out_str_P(AT_CPBW, GSM);
	ser_out_str(Index, GSM);
	ser_out_str(",\"", GSM);
	ser_out_str(Nummer, GSM);
	ser_out_str_P(nummer145 , GSM);
	ser_out_str(Titel, GSM);
	put_GSM("\"");
	delay();
		while (get_GSM((char*)&tmp, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp , OK) != 0) {return 0;}	
	return 1;
}

//Vor.: UART init und GSM init. Ein Index (1 - 230).
//Eff.: Der Eintrag ist aus dem Telefonbuch gelesen, falls nicht vorhanden return 2.
uint8_t read_entry(const char* Index, char* Nummer, char* Titel) {
	char tmp2[3] = "\0";
	char tmp[60] = "\0";
	uint8_t i = 0;
	uint8_t anf = 0;
	empty_ser_in();	
	ser_out_str_P(AT_CPBR, GSM);
	put_GSM(Index);
	delay();
	while (get_GSM((char*)&tmp, 60) == 0) {//auf Antwort warten
		;
	}
	if (strcmp_P(tmp , OK) == 0) {return 2;}//ist der Eintrag nicht vorhanden wird sofort ein OK gesendet
		
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 1. " erreicht
		i++;
	}
	i++;//eine Stelle nach 1. " weitermachen
	if (tmp[i] != 34) { //Nummer String vorhanden , ntig, da sonst die 2. suche misslingt
		anf = i;
		while ((tmp[i] != 34) && (i < 60)) {//String kopieren bis zum nchsten "
			Nummer[i-anf] = tmp[i];
			i++;
		}
		Nummer[i-anf] = '\0';
	}
	else {Nummer[0] = '\0';}
	i++;//eine Stelle nach 2. " weitermachen
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 3. " erreicht
		i++;
	}
	i++;//eine Stelle nach 3. " anfangen
	anf = i;
	while ((tmp[i] != 34) && (i < 60)) {//String kopieren bis zum nchsten "
		Titel[i-anf] = tmp[i];
		i++;
	}
	if ( i == 61) {
		Titel[0] = '\0';
		Nummer[0] = '\0';
		return 0;
	}
	Titel[i-anf] = '\0';
	return 1;
}

//***************************Datum/Uhrzeit*******************

//Vor.: UART init und GSM init.
//Eff.: Die Uhrzeit ist im Format hh:mm:ss geliefert.
uint8_t get_time(char* String) {
	char tmp[40] = "\0";
	char tmp2[3] = "\0";
	uint8_t i = 0;
	uint8_t anf = 0;
	empty_ser_in();
	put_GSM_P(AT_CCLK);
	delay();
	while (get_GSM((char*)&tmp, 40) == 0) {//auf Antwort warten
		;
	}	
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	while ((tmp[i] != ',') && (i < 40)) { //String durchlaufen bis , erreicht
		i++;
	}
	i++;//eine Stelle nach 1. " anfangen
	anf = i;
	while ((tmp[i] != '+') && (tmp[i] != '-') && (i < 40)) {//String kopieren bis zum nchsten +-
		String[i-anf] = tmp[i];
		i++;
	}
	if ( i == 21) {
		String[0] = '\0';
		return 0;
	}
	String[i-anf] = '\0';
	return 1;
}

//Vor.: UART init und GSM init.
//Eff.: Das Datum ist im Format yy/mm/dd geliefert.
uint8_t get_date(char* String) {
char tmp[40] = "\0";
	char tmp2[3] = "\0";
	uint8_t i = 0;
	uint8_t anf = 0;
	empty_ser_in();
	put_GSM_P(AT_CCLK);
	delay();
	while (get_GSM((char*)&tmp, 40) == 0) {//auf Antwort warten
		;
	}	
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	while ((tmp[i] != 34) && (i < 40)) { //String durchlaufen bis " erreicht
		i++;
	}
	i++;//eine Stelle nach 1. " anfangen
	anf = i;
	while ((tmp[i] != ',') && (i < 40)) {//String kopieren bis zum nchsten ,
		String[i-anf] = tmp[i];
		i++;
	}
	if ( i == 21) {
		String[0] = '\0';
		return 0;
	}
	String[i-anf] = '\0';
	return 1;
}	

//Vor.: UART init und GSM init. Uhreit/Datum String im Format yy/mm/dd,hh:mm:ss+00 .
//Eff.: Uhrzeit und Datum sind gespeichert.
uint8_t set_datetime(const char* String) {
	char tmp[3] = "\0";
	empty_ser_in();
	ser_out_str_P(AT_CCLK2, GSM);
	ser_out_str(String, GSM);
	put_GSM("\"");
	delay();
		while (get_GSM((char*)&tmp, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp , OK) != 0) {return 0;}	
	return 1;
}

//********************************SMS****************************

//Vor.: UART init und GSM init.
//Eff.: 1 ist geliefert bein ungelesenser SMS sonst 0
uint8_t get_newsms(void){
	uint8_t out = in_sms;
	if (out){
		in_sms = 0;
		return out;
	}
	return out;
}

//Vor.: UART init und GSM init.
//Eff.: Der Index der neuen SMS ist geliefert
uint8_t get_newsms_index(void){
	return sms_index[0];
}

//Vor.: UART init und GSM init.
//Eff.: Lscht alle SMS, folgende Modi:
//		READ
//		UNREAD
//		SENT
//		UNSENT
//		INBOX
//		ALL
uint8_t sms_deleteall(const char* mode) {
	char tmp[3] = "\0";
	empty_ser_in();
	ser_out_str_P(AT_CMGDA, GSM);
	put_GSM(mode);
	delay();
		while (get_GSM((char*)&tmp, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp , OK) != 0) {return 0;}	
	return 1;
}

//Vor.: UART init und GSM init, eine lschbare SMS.
//Eff.: Die SMS mit dem Index ist gelscht.
uint8_t sms_delete(const char* index) {
	char tmp[3] = "\0";
	empty_ser_in();
	ser_out_str_P(AT_CMGD, GSM);
	put_GSM(index);
	delay();
		while (get_GSM((char*)&tmp, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp , OK) != 0) {return 0;}	
	return 1;
}

//Vor.: UART init und GSM init, eine SMS.
//Eff.: Die Telefonnummer des Absenders ist geliefert, max 20 Zeichen.
uint8_t get_sms_nummer(const char* Index, char* Nummer) {
	char tmp[60] = "\0";
	char tmp2[3] = "\0";
	uint8_t i = 0;
	uint8_t anf = 0;
	uint8_t chara = 0;
	uint8_t count = 0;
	empty_ser_in();
	ser_out_str_P(AT_CMGR, GSM);
	put_GSM(Index);
	delay();
	while (get_GSM((char*)&tmp, 60) == 0) {//auf Antwort warten
		;
	}
	//SMS text verwerfen
	chara = uart_pollen(GSM);
	while (chara != 13) { //bis \r verwerfen
		count++;
		chara = uart_pollen(GSM);
	}
	chara = uart_pollen(GSM); // \n verwerfen
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 1. " erreicht
		i++;
	}
	i++;//eine Stelle nach 1. " weitermachen
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 2. " erreicht
		i++;
	}
	i++;//eine Stelle nach 2. " weitermachen
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 3. " erreicht
		i++;
	}
	i++;//eine Stelle nach 3. " anfangen
	anf = i;
	while ((tmp[i] != 34) && (i < 60)) {//String kopieren bis zum nchsten "
		Nummer[i-anf] = tmp[i];
		i++;
	}
	if ( i == 61) {
		Nummer[0] = '\0';
		return 0;
	}
	Nummer[i-anf] = '\0';
	return 1;
}

//Vor.: UART init und GSM init, eine SMS.
//Eff.: Der Absenderist geliefert insofern er im Telefonbuch steht, max 20 Zeichen.
//      Eintrag im Telefonbuch nicht enthalten -> leerer String.
uint8_t get_sms_absender(const char* Index, char* Absender) {
	char tmp[61] = "\0";
	char tmp2[3] = "\0";
	uint8_t i = 0;
	uint8_t anf = 0;
	uint8_t chara = 0;
	uint8_t count = 0;
	empty_ser_in();
	ser_out_str_P(AT_CMGR, GSM);
	put_GSM(Index);
	delay();
	while (get_GSM((char*)&tmp, 60) == 0) {//auf Antwort warten
		;
	}
	//SMS text verwerfen
	chara = uart_pollen(GSM);
	while ((chara != 13) && (chara != 0)) { //bis \r verwerfen
		count++;
		chara = uart_pollen(GSM);
	}
	chara = uart_pollen(GSM); // \n verwerfen
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 1. " erreicht
		i++;
	}
	i++;//eine Stelle nach 1. " weitermachen
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 2. " erreicht
		i++;
	}
	i++;//eine Stelle nach 2. " weitermachen
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 3. " erreicht
		i++;
	}
	i++;//eine Stelle nach 3. " anfangen
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 4. " erreicht
		i++;
	}
	i++;//eine Stelle nach 4. " anfangen
	while ((tmp[i] != 34) && (i < 60)) { //String durchlaufen bis 5. " erreicht
		i++;
	}
	i++;//eine Stelle nach 5. " anfangen
	anf = i;
	while ((tmp[i] != 34) && (i < 60)) {//String kopieren bis zum nchsten "
		Absender[i-anf] = tmp[i];
		i++;
	}
	if ( i == 61) {
		Absender[0] = '\0';
		return 0;
	}
	Absender[i-anf] = '\0';
	return 1;
}

uint8_t send_sms(const char* nummer, const char* text){

	char tmp[30] = "\0";
	char tmp2[3] = "\0";
	uint8_t i = 0;
	uint8_t anf;
	
	empty_ser_in();
	ser_out_str_P(AT_CMGS, GSM);
	ser_out('\"', GSM);
	ser_out_str(nummer, GSM);
	ser_out_str("\"\r", GSM);
	//auf ">" warten
	while ('>' != uart_pollen(GSM));
	ser_out_str(text, GSM);
	ser_out(26, GSM);
	ser_out(13, GSM);
	ser_out(10, GSM);
	
	while (get_GSM((char*)&tmp, 30) == 0) {//auf Antwort warten
		;
	}

	while(tmp[i] != 32){
		if(tmp[i] == 'E'){return 0;}
		i++;
	}

	//Speicherplatznummer
	i++;
	anf = i;
	
	while(tmp[i] != 0){
		tmp[i-anf] = tmp[i];
		i++;
	}
	tmp[i-anf] = 0;
	
	//OK
	while (get_GSM((char*)&tmp2, 3) == 0) {//auf Besttigung warten
		;
	}
	if (strcmp_P(tmp2 , OK) != 0) {return 0;}
	
	return atoi(tmp);
}