#ifndef _GSM_H_
#define _GSM_H_

#define UBRR_REG 	1000 //UBRR Registerinhalt fr 38400baud beim Systemtakt
#define PWRKEY_PORT PORTC
#define PWRKEY_PIN	PC0
#define STATUS_PORT PORTC
#define STATUS_PIN	PC1
#define DDR(x) (*(&x - 1))
#define PIN(x) (*(&x - 2))
#define GSM 1

//Deletemodi
#define READ "DEL READ"
#define UNREAD "DEL UNREAD"
#define SENT "DEL SENT"
#define UNSENT "DEL UNSENT"
#define INBOX "DEL INBOX"
#define ALL "DEL ALL"

//Prozedur zum einbinden anderer UART Librarys
uint16_t ser_in(uint8_t ziel);

/** \brief liet den Input vom GSM Modul und bereitet diesen auf

	Falls vorhanden ist der String vom GSM Modul geliefert, ohne \r\n. <br>
	RING, NO_CARRIER und eingehende SMS (CMTI) werden herausgefiltert. <br>
	Die Statusvariablen sind entsprechend gesetzt.
	
	\param String	Buffer zum speichern des Strings vom GSM Modul
	\param maxchar	maximale Lnge des Strings

	\return 1 wenn ein String gelesen wurde, sonst 0
*/
uint8_t get_GSM(char *String, uint8_t maxchar);

/** \brief leert den UART Empfangsbuffer
*/
void empty_ser_in(void);

/** \brief initialisierung des GSM Moduls
*/
void init_GSM(void);

/** \brief GSM Modul abschalten
*/
void off_GSM(void);

//***************************Profile Commands******************************

/** \brief prft ob das GSM Modul auf dem UART Befehle empfngt

	\return 1 wenn AT Interpreter des GSM Modul aktiv
	\return 0 wenn AT Interpreter inaktiv
*/
uint8_t AT_active(void);

/** \brief liet den Namen des GSM Modul aus

	\param String 	Pointer auf Buffer fr den GSM Modul Namen
	\param maxchar 	maximale Lnge des Strings
	
	\return 1 bei Erfolg
*/
uint8_t GSM_Modul(char *String, uint8_t maxchar);

/** \brief liet die Empfangsstrke aus

	0 - weniger als -115dBm <br>
	1 - -111dBm <br>
	2...30 - -110...-54dBm <br>
	31 - mehr als -52dBm <br>
	99 - unbekannt

	\param String Pointer auf Buffer der Lnge 3 
	
	\return 1 bei Erfolg
*/
uint8_t Empfang(char *String);


//**************************General Commands*******************************

/** \brief liet den Netzstatus aus

	0: fehlgeschlagen <br>
	1: eingebucht, Heimatnetz <br>
	2: nicht eingebucht, Netzsuche <br>
	3: nicht eingebucht, Einbuchung abgelehnt <br>
	4: Status unbekannt <br>
	5: eingebucht, Fremdnetz
	
	\return 1 bei Erfolg
*/
uint8_t Netzstatus (void);

/** \brief liet den Providernamen aus

	\param String ausreichend langer Buffer fr den Namen
	
	\return 1 bei Erfolg
*/
uint8_t Provider(char *String);

//****************************Telefoncommands***************

/** \brief hat jemand angerufen?

	\return 1 bei anruf, sonst 0
*/
uint8_t get_ring(void);

/** \brief Rufnummer des momentanen Anrufers auslesen

	Der String ist leer	bei unterdrckter Nummer. Maximale lnge: 20 zeichen.
	
	\param String Buffer mit der Lnge 21
	
	\return 1 bei Erfolg
*/
uint8_t get_ringingnumber(char *String);

/** \brief Telefonbuchname des momentanen Anrufers

	Der Name des Anrufers ist geliefert, insofern er im Telefonbuch steht. 
	Der String ist leer	bei keinem auffindbaren Eintrag. Maximale lnge: 16 zeichen.
	
	\param String Buffer mit der Lnge 17
	
	\return 1 bei Erfolg
*/
uint8_t get_anrufer(char *String);

/** \brief auflegen des momentanen anrufs

	\return 1 bei Erfolg
*/
uint8_t auflegen(void);

/** \brief annehmen des momentanen anrufs

	\return 1 bei Erfolg
*/
uint8_t annehmen(void);

/** \brief einen Anruf ttigen

	\param Nummer  String im Format +49xxxx
	
	\return 1 bei Erfolg
*/
uint8_t anrufen(char* Nummer);


//*************************Telefonbuch*******************

/** \brief einen EIntrag im SIM Telefonbuch anlegen
	 
	\param Index 	Der Telefonbuchindex in den geschrieben werden soll (1 - 230)
	\param Nummer 	Die Nummer die geschrieben werden soll im Format "145" (+49xxx), max 20 Zeichen
	\param Titel 	Der Name des Eintrags, max 16 Zeichen
	
	\return 1 bei Erfolg
*/
uint8_t write_entry(const char* Index, const char* Nummer, const char * Titel);

/** \brief einen Eintrag aus dem SIM Telefonbuch lesen

	\param Index 	Der Telefonbuchindex der gelesen werden soll (1 - 230)
	\param Nummer 	Buffer in den die Nummer im Format "145" (+49xxx) geschrieben wird, max 20 Zeichen
	\param Titel 	Buffer in den der Name des Eintrags geschrieben wird, max 16 Zeichen
	
	\return 1 bei Erfolg
*/
uint8_t read_entry(const char* Index, char* Nummer, char* Titel);


//***************************Datum/Uhrzeit*******************

/** \brief Uhrzeit aus der RTC des SIM900D auslesen

	\param String Buffer in dem die Zeit im Format hh:mm:ss gespeichert wird
	
	\return 1 bei Erfolg
*/
uint8_t get_time(char* String);

/** \brief Datum aus der RTC des SIM900D auslesen

	\param String Buffer in dem das Datum im Format yy/mm/dd gespeichert wird
	
	\return 1 bei Erfolg
*/
uint8_t get_date(char* String);

/** \brief setzt Uhrzeit und Datum der RTC des SIM900D

	\param String Zeit und Datum im Format yy/mm/dd,hh:mm:ss+00
	
	\return 1 bei Erfolg
*/
uint8_t set_datetime(const char* String);


//********************************SMS****************************

/** \brief sind neue SMS eingetroffen?

	\return 1 bei neuer SMS, sonst 0
*/
uint8_t get_newsms(void);

/** \brief auf welchem SIM Speicherplatz liegt die neue SMS?
	
	\return Der Index der neuen SMS
*/
uint8_t get_newsms_index(void);

/** \brief lschen von ganzen SMS Gruppen

	READ <br>
	UNREAD <br>
	SENT <br>
	UNSENT <br>
	INBOX <br>
	ALL <br>
	
	\param mode einer der oben genannten
	
	\return 1 bei Erfolg
*/
uint8_t sms_deleteall(const char* mode);

//Vor.: UART init und GSM init, eine lschbare SMS.
//Eff.: Die SMS mit dem Index ist gelscht.
/** \brief lschen einer SMS

	\param index der Index des SIM Speichers der zu lschenden SMS
	
	\return 1 bei Erfolg
*/
uint8_t sms_delete(const char* index);

/** \brief liet die Nummer des SMS Absenders aus

	\param index 	der Index des SIM Speichers
	\param Nummer	Buffer zum speichern der Nummer, max 20 Zeichen
	
	\return 1 bei Erfolg
*/
uint8_t get_sms_nummer(const char* Index, char* Nummer);

/** \brief liet den Absender der SMS aus dem SIM Telefonbuch aus

	Eintrag im Telefonbuch nicht enthalten -> leerer String.

	\param Index 	der Index des SIM Speichers
	\param Absender Buffer fr den Absender, max 20 Zeichen
	
	\return 1 bei Erfolg
*/
uint8_t get_sms_absender(const char* Index, char* Absender);

//return: Speicherplatz
/** \brief sendet eine SMS

	\param nummer 	String im Format +49xxxx
	\param text		Text der gesendet werden soll, max 160 Zeichen
	
	\return Speicherplatz Index der gesendet SMS
	\return 0 bei Sendefehler
*/
uint8_t send_sms(const char* nummer, const char* text);

#endif//_GSM_H_